/* 3D Cylinder */

// Parameters
// domain
DefineConstant[ dmt = { 1.0, Name "Cylinder diameter" }  ];
DefineConstant[ spn = { 2.5, Name "Cylinder span" }  ];
DefineConstant[ tpt = { .05, Name "Cylinder tip thickness" }  ]; // percentage of span
DefineConstant[ lgt = { 8.0, Name "Channel length" }  ];
DefineConstant[ wdt = { 4.0, Name "Channel width" }  ];
DefineConstant[ hgt = { 6.0, Name "Channel height" }  ];
DefineConstant[ msN = { 0.1, Min 0.1, Max 1, Step 0.1, Name "Nearfield mesh size" }  ];
DefineConstant[ msF = { 1.0, Min 0.1, Max 1, Step 0.1, Name "Farfield mesh size" }  ];

// mesh algo
msh3D = 2;
msh2D = 5;

// Geometry
// cylinder
Point(0) = {0, 0, 0};
Point(1) = {dmt/2, 0, 0, msN};
Point(2) = {0, 0, dmt/2, msN};
Point(3) = {-dmt/2, 0, 0, msN};
Point(4) = {0, 0, -dmt/2, msN};
Point(10) = {0, spn, 0};
Point(11) = {dmt/2, spn, 0, msN};
Point(12) = {0, spn, dmt/2, msN};
Point(13) = {-dmt/2, spn, 0, msN};
Point(14) = {0, spn, -dmt/2, msN};

// tip
Point(21) = {0, spn+tpt*spn, 0, msN};

// box
Point(31) = {lgt/2, 0, hgt/2, msF};
Point(32) = {-lgt/2, 0, hgt/2, msF};
Point(33) = {-lgt/2, 0, -hgt/2, msF};
Point(34) = {lgt/2, 0, -hgt/2, msF};
Point(35) = {lgt/2, wdt, hgt/2, msF};
Point(36) = {-lgt/2, wdt, hgt/2, msF};
Point(37) = {-lgt/2, wdt, -hgt/2, msF};
Point(38) = {lgt/2, wdt, -hgt/2, msF};

// midplane
Point(41) = {lgt/2, 0, 0, msF};
Point(42) = {lgt/2, spn, 0, msF};
Point(43) = {lgt/2, wdt, 0, msF};
Point(44) = {dmt/2, wdt, 0, msF};
Point(45) = {0, wdt, 0, msF};
Point(46) = {-dmt/2, wdt, 0, msF};
Point(47) = {-lgt/2, wdt, 0, msF};
Point(48) = {-lgt/2, spn, 0, msF};
Point(49) = {-lgt/2, 0, 0, msF};

// wing
Circle(1) = {1, 0, 2};
Circle(2) = {2, 0, 3};
Circle(3) = {3, 0, 4};
Circle(4) = {4, 0, 1};
Circle(11) = {11, 10, 12};
Circle(12) = {12, 10, 13};
Circle(13) = {13, 10, 14};
Circle(14) = {14, 10, 11};
Line(21) = {1, 11};
Line(22) = {2, 12};
Line(23) = {3, 13};
Line(24) = {4, 14};

// tip
Ellipse(31) = {11, 10, 10, 21};
Ellipse(32) = {12, 10, 10, 21};
Ellipse(33) = {13, 10, 10, 21};
Ellipse(34) = {14, 10, 10, 21};

// box
Line(41) = {41, 31};
Line(42) = {31, 32};
Line(43) = {32, 49};
Line(44) = {49, 33};
Line(45) = {33, 34};
Line(46) = {34, 41};
Line(47) = {43, 35};
Line(48) = {35, 36};
Line(49) = {36, 47};
Line(50) = {47, 37};
Line(51) = {37, 38};
Line(52) = {38, 43};
Line(53) = {31, 35};
Line(54) = {32, 36};
Line(55) = {33, 37};
Line(56) = {34, 38};

// midplane
Line(61) = {1, 41};
Line(62) = {11, 42};
Line(63) = {11, 44};
Line(64) = {21, 45};
Line(65) = {13, 46};
Line(66) = {13, 48};
Line(67) = {3, 49};
Line(68) = {41, 42};
Line(69) = {42, 43};
Line(70) = {43, 44};
Line(71) = {44, 45};
Line(72) = {45, 46};
Line(73) = {46, 47};
Line(74) = {47, 48};
Line(75) = {48, 49};

// cylinder
Line Loop(1) = {1, 22, -11, -21};
Surface(1) = {-1};
Line Loop(2) = {2, 23, -12, -22};
Surface(2) = {-2};
Line Loop(3) = {3, 24, -13, -23};
Surface(3) = {-3};
Line Loop(4) = {4, 21, -14, -24};
Surface(4) = {-4};
//tip
Line Loop(5) = {11, 32, -31};
Surface(5) = {-5};
Line Loop(6) = {12, 33, -32};
Surface(6) = {-6};
Line Loop(7) = {13, 34, -33};
Surface(7) = {-7};
Line Loop(8) = {14, 31, -34};
Surface(8) = {-8};

// symmetry
Line Loop(9) = {61, 41, 42, 43, -67, -2, -1};
Plane Surface(9) = {-9};
Line Loop(10) = {4, 61, -46, -45, -44, -67, 3};
Plane Surface(10) = {10};

// upstream
Line Loop(11) = {43, -75, -74, -49, -54};
Plane Surface(11) = {11};
Line Loop(12) = {75, 44, 55, -50, 74};
Plane Surface(12) = {12};

// downstream
Line Loop(13) = {41, 53, -47, -69, -68};
Plane Surface(13) = {13};
Line Loop(14) = {68, 69, -52, -56, 46};
Plane Surface(14) = {14};

// farfield
Line Loop(15) = {42, 54, -48, -53};
Plane Surface(15) = {15};
Line Loop(16) = {47, 48, 49, -73, -72, -71, -70};
Plane Surface(16) = {16};
Line Loop(17) = {51, 52, 70, 71, 72, 73, 50};
Plane Surface(17) = {17};
Line Loop(18) = {45, 56, -51, -55};
Plane Surface(18) = {18};

// wake
Line Loop(19) = {61, 68, -62, -21};
Surface(19) = {19};

// internal
Line Loop(20) = {62, 69, 70, -63};
Surface(20) = {20};
Line Loop(21) = {64, -71, -63, 31};
Surface(21) = {-21};
Line Loop(22) = {65, -72, -64, -33};
Surface(22) = {-22};
Line Loop(23) = {74, -66, 65, 73};
Surface(23) = {23};
Line Loop(24) = {75, -67, 23, 66};
Surface(24) = {24};

// field
Surface Loop(1) = {15, 9, 13, 16, 11, 2, 6, 5, 1, 24, 23, 22, 21, 20, 19};
Volume(1) = {1};
Surface Loop(2) = {10, 4, 8, 7, 3, 14, 17, 18, 12, 24, 23, 22, 21, 20, 19};
Volume(2) = {2};

// Mesh
// cylinder, farfield, symmetry, midplane
MeshAlgorithm Surface{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24} = msh2D;
// 3D
Mesh.Algorithm3D = msh3D;
Mesh.OptimizeNetgen = 1;
Mesh.Smoothing = 10;
Mesh.SmoothNormals = 1;

// Physical
Physical Line("teTip") = {21, 62};
Physical Line("wakeTip") = {62};
Physical Surface("cylinder") = {1, 2};
Physical Surface("cylinder_") = {3, 4};
Physical Surface("tip") = {5, 6, 7, 8};
Physical Surface("symmetry") = {9};
Physical Surface("symmetry_") = {10};
Physical Surface("upstream") = {11, 12};
Physical Surface("downstream") = {13};
Physical Surface("downstream_") = {14};
Physical Surface("farfield") = {15, 16, 17, 18};
Physical Surface("wake") = {19};
Physical Volume("field") = {1};
Physical Volume("field_") = {2};

